package se.jbee.inject.bind;

import static org.hamcrest.CoreMatchers.instanceOf;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static se.jbee.inject.Dependency.dependency;

import org.junit.Test;

import se.jbee.inject.Injector;
import se.jbee.inject.bootstrap.Bootstrap;

/**
 * The test demonstrates binds that are 'linked' by type.
 * 
 * @author Jan Bernitt (jan@jbee.se)
 */
public class TestInstanceBinds {

	private static class Foo {

	}

	private static class InstanceBindsModule
			extends BinderModule {

		@Override
		protected void declare() {
			bind( Number.class ).to( Integer.class );
			bind( Integer.class ).to( 42 );
			bind( Foo.class ).to( Foo.class );
		}

	}

	private final Injector injector = Bootstrap.injector( InstanceBindsModule.class );

	@Test
	public void thatNumberDependencyIsResolvedToIntegerBoundSupplier() {
		Number number = injector.resolve( dependency( Number.class ) );
		assertThat( number, instanceOf( Integer.class ) );
		assertThat( number.intValue(), is( 42 ) );
	}

	@Test
	public void thatTypeLinkedBackToItselfBecomesConstructorBinding() {
		assertNotNull( injector.resolve( dependency( Foo.class ) ) );
	}
}
